@@ -7,6 +7,10 @@ try:
     _websocket_available = True
 except ImportError:
     _websocket_available = False
+try:
+    import uwsgi
+except ImportError:
+    uwsgi = None
 
 
 class Thread(gevent.Greenlet):  # pragma: no cover
@@ -30,22 +34,38 @@ class WebSocketWSGI(object):  # pragma: no cover
         self.app = app
 
     def __call__(self, environ, start_response):
-
-        self._sock = environ["wsgi.websocket"]
         self.environ = environ
-        self.version = self._sock.version
-        self.path = self._sock.path
-        self.origin = self._sock.origin
-        self.protocol = self._sock.protocol
+        if uwsgi and 'uwsgi.version' in environ:
+            # we are running under uwsgi
+            uwsgi.websocket_handshake(environ['HTTP_SEC_WEBSOCKET_KEY'],
+                                      environ.get('HTTP_ORIGIN', ''))
+            self._sock = None
+        elif 'wsgi.websocket' in environ:
+            self._sock = environ['wsgi.websocket']
+            #self.version = self._sock.version
+            #self.path = self._sock.path
+            #self.origin = self._sock.origin
+            #self.protocol = self._sock.protocol
+        else:
+            raise RuntimeError('You need to use the gevent-websocket server. '
+                               'See the Deployment section of the '
+                               'documentation for more information.')
         return self.app(self)
 
     def close(self):
-        return self._sock.close()
+        if self._sock:
+            return self._sock.close()
 
     def send(self, message):
+        print("send", message)
+        if self._sock is None:
+            return uwsgi.websocket_send(message)
         return self._sock.send(message)
 
     def wait(self):
+        print("reading")
+        if self._sock is None:
+            return uwsgi.websocket_recv()
         return self._sock.receive()
 
 
